#
#  Copyright 1999 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc create_filter_window { record_list } {

	set toplevel .filter_window

	set status [ catch { toplevel $toplevel } ]

	# If the window already exists, bring it to the top.

	if { $status != 0 } {
		raise $toplevel
		focus $toplevel
		return
	}

	wm title $toplevel "Filter Control"

	set grid $toplevel.grid

	frame $grid
	pack $grid -side top

	# Create the filter control grid top labels.

	label $grid.current -text "Current"
	label $grid.desired -text "Desired"

	grid $grid.current -row 0 -column 1 
	grid $grid.desired -row 0 -column 3

	# Create the filter row.

	label $grid.filter_label -text "Thickness"
	label $grid.filter_current -fg blue -text "---"
	label $grid.filter_current_units -text "um"

	iwidgets::optionmenu $grid.filter_desired

	set sorted_thickness_list [ lsort -real $IMCAGui::thickness_list ]

	foreach item $sorted_thickness_list {
		$grid.filter_desired insert end $item
	}

	set thickness [ get_filter_thickness ]

	if { $thickness == "Moving" } {
		set thickness 0
	}
	if { $thickness == "Fault" } {
		set thickness 0
	}
	set selection [ lsearch -exact $sorted_thickness_list $thickness ]

	if { $selection < 0 } {
		set selection 0
	} 

	$grid.filter_desired select $selection

	label $grid.filter_desired_units -text "um"

	grid $grid.filter_label         -row 1 -column 0
	grid $grid.filter_current       -row 1 -column 1
	grid $grid.filter_current_units -row 1 -column 2 -sticky w
	grid $grid.filter_desired       -row 1 -column 3
	grid $grid.filter_desired_units -row 1 -column 4 -sticky w

	frame $toplevel.sep -height 2 -borderwidth 1 -relief sunken
	pack $toplevel.sep -expand yes -fill x -pady 5 -side top

	# Create the bottom row.

	set bottom $toplevel.bottom

	frame $bottom
	pack $bottom -side top

	button $bottom.move -text "Move" \
		-command [ code move_filters $record_list $grid ]

	pack $bottom.move -side left

	frame $bottom.sep
	pack $bottom.sep -side left -padx 5

	button $bottom.dismiss -text "Dismiss" \
		-command [ code destroy $toplevel ]

	pack $bottom.dismiss -side left

	add_monochromator_callback filter_window_callback
}

proc move_filters { record_list grid } {

	set filter1 [ $record_list get_record "filter1" ]
	set filter2 [ $record_list get_record "filter2" ]

	set thickness_desired [ $grid.filter_desired get ]

	set i [ lsearch -exact $IMCAGui::thickness_list $thickness_desired ]

	if { $i < 0 } {
		set t "Filter thickness $thickness_desired not found in "
		append t "filter thickness list."

		tk_dialog .bad_filter_thickness_dialog \
		"Bad Filter Thickness" $t error 0 "OK"

		return
	}

	set index_list [ lindex $IMCAGui::thickness_indices $i ]

	set index1 [ lindex $index_list 0 ]
	set index2 [ lindex $index_list 1 ]

	set filter1_position $IMCAGui::filter1_position($index1)
	set filter2_position $IMCAGui::filter2_position($index2)

	$filter1 move_absolute $filter1_position
	$filter2 move_absolute $filter2_position
}

proc filter_window_callback {} {

	set window .filter_window.grid.filter_current

	if { [ winfo exists $window ] == 0 } {
		delete_monochromator_callback filter_window_callback
		return
	}

	set thickness [ get_filter_thickness ]

	$window configure -text $thickness

	if { $thickness == "Fault" } {
		$window configure -fg red
	} elseif { $thickness == "Moving" } {
		$window configure -fg limegreen
	} else {
		$window configure -fg blue
	}
}

proc get_filter_thickness {} {

	global callback_values
	global callback_records

	if [ $callback_records(filter1) is_busy ] {
		return "Moving"
	}
	if [ $callback_records(filter2) is_busy ] {
		return "Moving"
	}

	set threshold 0.1

	set position1 $callback_values(filter1)
	set position2 $callback_values(filter2)

	set thickness1 -1
	set thickness2 -1

	set nelements [ array size IMCAGui::filter1_position ]

	for { set i 0 } { $i < $nelements } { incr i } {

		set item_position $IMCAGui::filter1_position($i)

		set position_diff \
			[ expr ( abs( $item_position - $position1 ) ) ]

		if { $position_diff < $threshold } {
			set thickness1 $IMCAGui::filter1_thickness($i)
			break
		}
	}

	if { $thickness1 < 0 } {
		return "Fault"
	}

	set nelements [ array size IMCAGui::filter2_position ]

	for { set i 0 } { $i < $nelements } { incr i } {

		set item_position $IMCAGui::filter2_position($i)

		set position_diff \
			[ expr ( abs( $item_position - $position2 ) ) ]

		if { $position_diff < $threshold } {
			set thickness2 $IMCAGui::filter2_thickness($i)
			break
		}
	}

	if { $thickness2 < 0 } {
		return "Fault"
	}

	set result [ expr ( $thickness1 + $thickness2 ) ]

	return $result
}
