#
#  Copyright 1999 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc create_imcagui_gui { record_list } {

	create_imcagui_menubar $record_list

	create_imcagui_main_window $record_list

	setup_monochromator_callbacks $record_list
}

proc setup_monochromator_callbacks { record_list } {

	find_callback_record $record_list "lambda"
	find_callback_record $record_list "d_spacing"
	find_callback_record $record_list "bh_size"
	find_callback_record $record_list "bv_size"

	if { $IMCAGui::program_name == "imca17id" } {
		find_callback_record $record_list "id_ev"
		find_callback_record $record_list "id_ev_harmonic"
		find_callback_record $record_list "filter1"
		find_callback_record $record_list "filter2"
	}

	after 1 [ list process_monochromator_callbacks ]
}

proc find_callback_record { record_list record_name } {

	global callback_records

	set status [ catch { $record_list get_record $record_name } \
			record ]

	if { $status != 0 } {
		log_puts \
		"Error: '$record_name' record not found in database."

		return -code error
	}

	set callback_records($record_name) $record
}

proc process_monochromator_callbacks { } {

	global callback_list
	global callback_records
	global callback_values

	set callback_values(lambda) \
		[ $callback_records(lambda) get_position ]

	set callback_values(energy) \
		[ energy_from_lambda $callback_values(lambda) ]

	set callback_values(d_spacing) \
		[ $callback_records(d_spacing) receive_variable ]

	set callback_values(bh_size) \
		[ $callback_records(bh_size) get_position ]

	set callback_values(bv_size) \
		[ $callback_records(bv_size) get_position ]

	if { $IMCAGui::program_name == "imca17id" } {
		set callback_values(id_ev) \
			[ $callback_records(id_ev) get_position ]

		set callback_values(id_ev_harmonic) \
			[ $callback_records(id_ev_harmonic) receive_variable ]

		set callback_values(filter1) \
			[ $callback_records(filter1) get_position ]

		set callback_values(filter2) \
			[ $callback_records(filter2) get_position ]
	}

	foreach callback $callback_list {
		$callback
	}

	after 1000 [ list process_monochromator_callbacks ]
}

proc add_monochromator_callback { callback } {
	global callback_list

	lappend callback_list $callback
}

proc delete_monochromator_callback { callback } {
	global callback_list

	set i [ lsearch -exact $callback_list $callback ]

	if { $i < 0 } {
		set t \
	    "Attempt to delete callback '$callback' from callback list failed."

		tk_dialog .delete_callback_failed \
		"Callback Deletion Failed." $t error 0 "OK"

		return
	}

	set callback_list [ lreplace $callback_list $i $i ]
}

proc create_imcagui_menubar { record_list } {
	menu .menubar
	. configure -menu .menubar

	###############################################################
	#
	#   Add the File Menu
	#

	menu .menubar.file -tearoff 0
	.menubar add cascade -label File -menu .menubar.file

	.menubar.file add command -label Print -command [ code print_window . ]

	.menubar.file add command -label Staff \
		-command [ code eval exec "mxgui &" ]

	.menubar.file add command -label Exit -command exit

	###############################################################
	#
	#   Add the Spectroscopy / MAD Menu
	#

	menu .menubar.mad -tearoff 0
	.menubar add cascade -label Experiment -menu .menubar.mad

	.menubar.mad add command -label Spectroscopy -command [ code \
		create_experiment_window Spectroscopy $record_list ]

	.menubar.mad add command -label MAD -command [ code \
		create_experiment_window MAD $record_list ]

	###############################################################
	#
	#   Add the Help Menu
	#

	menu .menubar.help -tearoff 0
	.menubar add cascade -label Help -menu .menubar.help -state disabled

	.menubar.help add command -label "Placeholder for Help"
}

#
#  Create the main window.
#
proc create_imcagui_main_window { record_list } {

	set window .main

	frame .main
	pack .main -before .message_log

	#-----

	set frame .main.energy_frame

	frame $frame
	pack $frame -side top

	frame $frame.grid
	pack $frame.grid -side left

	create_imcagui_energy_frame $record_list $frame.grid

	frame $frame.sep
	pack $frame.sep -side left -padx 5

	button $frame.move_button -text Move -command \
			[ code move_lambda $frame.grid ]

	pack $frame.move_button -side left

	#-----

	frame .main.sep1 -height 2 -borderwidth 1 -relief sunken
	pack .main.sep1 -expand yes -fill x -pady 5 -side top

	#-----

	set frame .main.harmonic_frame

	frame $frame
	pack $frame -side top

	create_harmonic_frame $frame

	#-----

	frame .main.sep2 -height 2 -borderwidth 1 -relief sunken
	pack .main.sep2 -expand yes -fill x -pady 5 -side top

	#-----

	set slit_frame .main.slit_frame

	frame $slit_frame
	pack $slit_frame -side top

	label $slit_frame.label -text "Slits:   "
	pack $slit_frame.label -side left

	set center_frame $slit_frame.main

	frame $center_frame
	pack $center_frame -side left

	create_slit_frame $center_frame

	frame $slit_frame.sep
	pack $slit_frame.sep -side left -padx 5

	button $slit_frame.move_button -text Move -command \
			[ code move_slits $center_frame ]

	pack $slit_frame.move_button -side left

	#-----

	frame .main.sep3 -height 2 -borderwidth 1 -relief sunken
	pack .main.sep3 -expand yes -fill x -pady 5 -side top

	#-----

	set frame .main.bottom

	frame $frame
	pack $frame -side top

	button $frame.filters -text "Filter Control..." \
		-command [ code create_filter_window $record_list ]

	pack $frame.filters -side left

	if { $IMCAGui::program_name != "imca17id" } {

		$frame.filters configure -state disabled
	}

	frame $frame.sep
	pack $frame.sep -side left -padx 5

	set command "optimize_gui &"

	button $frame.optimize -text "Optimize Intensity..." \
		-command [ code eval exec $command ]

	pack $frame.optimize -side left

	#-----

	frame .main.sep4 -height 2 -borderwidth 1 -relief sunken
	pack .main.sep4 -expand yes -fill x -pady 5 -side top

	#-----

	add_monochromator_callback main_window_callback
}

proc main_window_callback { } {

	global callback_values

	set window .main

	set frame $window.energy_frame.grid

	$frame.energy_value configure -text \
		[ format "%10.3f" $callback_values(energy) ]

	$frame.lambda_value configure -text \
		[ format "%8.4f"  $callback_values(lambda) ]

	if { $IMCAGui::program_name == "imca17id" } {

		$frame.id_value configure -text \
			[ format "%10.3f" $callback_values(id_ev) ]
	}

	set frame $window.harmonic_frame

	if { $IMCAGui::program_name == "imca17id" } {

		$frame.harmonic_value configure -text \
			[ format "%2d" $callback_values(id_ev_harmonic) ]
	}

	$frame.d_spacing_value configure -text \
		[ format "%8.5f" $callback_values(d_spacing) ]

	set frame $window.slit_frame.main

	$frame.horiz_value configure -text \
		[ format "%10.3f" $callback_values(bh_size) ]

	$frame.vert_value configure -text \
		[ format "%10.3f" $callback_values(bv_size) ]
}

############################################################################
#
#  Create the energy and wavelength display frame.
#
proc create_imcagui_energy_frame { record_list frame } {

	set row 0

	label $frame.current_label -text Current -width 10
	label $frame.desired_label -text Desired 

	grid $frame.current_label -row $row -column 1
	grid $frame.desired_label -row $row -column 3

	incr row

	label $frame.energy_label -text Energy
	label $frame.energy_value -text "---" -fg blue
	label $frame.energy_units1 -text "eV   "
	label $frame.energy_units2 -text eV

	iwidgets::entryfield $frame.energy_desired -width 10 \
		-validate { iwidgets::Entryfield::real %P } \
		-command [ code energy_desired_update $frame ]

	grid $frame.energy_label   -row $row -column 0 -sticky w
	grid $frame.energy_value   -row $row -column 1 -sticky e
	grid $frame.energy_units1  -row $row -column 2 -sticky w
	grid $frame.energy_desired -row $row -column 3
	grid $frame.energy_units2  -row $row -column 4 -sticky w

	incr row

	label $frame.lambda_label -text Wavelength
	label $frame.lambda_value -text "---" -fg blue
	label $frame.lambda_units1 -text A
	label $frame.lambda_units2 -text A

	iwidgets::entryfield $frame.lambda_desired -width 10 \
		-validate { iwidgets::Entryfield::real %P } \
		-command [ code lambda_desired_update $frame ]

	grid $frame.lambda_label   -row $row -column 0 -sticky w
	grid $frame.lambda_value   -row $row -column 1 -sticky e
	grid $frame.lambda_units1  -row $row -column 2 -sticky w
	grid $frame.lambda_desired -row $row -column 3
	grid $frame.lambda_units2  -row $row -column 4 -sticky w

	incr row

	label $frame.id_label -text "Gap Energy"
	label $frame.id_value -text "---" -fg blue
	label $frame.id_units -text eV

	grid $frame.id_label -row $row -column 0 -sticky w
	grid $frame.id_value -row $row -column 1 -sticky e
	grid $frame.id_units -row $row -column 2 -sticky w
}

proc energy_desired_update { frame } {

	set energy [ $frame.energy_desired get ]

	set lambda [ lambda_from_energy $energy ]

	$frame.lambda_desired clear
	$frame.lambda_desired insert end $lambda
}

proc lambda_desired_update { frame } {

	set lambda [ $frame.lambda_desired get ]

	set energy [ energy_from_lambda $lambda ]

	$frame.energy_desired clear
	$frame.energy_desired insert end $energy
}

proc move_lambda { frame } {

	global callback_records
	global callback_values

	set lambda_requested [ $frame.lambda_desired get ]

	if { $lambda_requested == "" } {

		set energy_requested [ $frame.energy_desired get ]

		if { $energy_requested == "" } {
			set t "No energy or wavelength value requested."

			tk_dialog .no_wavelength_dialog \
				"No Value Specified" $t error 0 "OK"

			return
		}

		set lambda_requested \
			[ lambda_from_energy $energy_requested ]
	}

	set status [ catch { expr $lambda_requested } junk ]

	if { $status != 0 } {
		set t "Invalid wavelength value '$lambda_requested' requested."

		tk_dialog .invalid_wavelength_dialog \
			"Invalid Wavelength" $t error 0 "OK"

		return
	}

#	if [ expr { $lambda_requested > 1.16 } ] {
#		$callback_records(id_ev_harmonic) send_variable 1
#	} else {
#		$callback_records(id_ev_harmonic) send_variable 3
#	}

	$callback_records(lambda) move_absolute $lambda_requested
}

############################################################################

proc create_harmonic_frame { frame } {

	label $frame.harmonic_label -text "Undulator\nHarmonic" 
	pack $frame.harmonic_label -side left

	frame $frame.sep1 -width 1 -borderwidth 1
	pack $frame.sep1 -side left -expand yes -fill y -padx 5

	label $frame.harmonic_value -text "-" -fg blue
	pack $frame.harmonic_value -side left

	frame $frame.sep2 -width 1 -borderwidth 1
	pack $frame.sep2 -side left -expand yes -fill y -padx 20

	label $frame.d_spacing_label -text "Monochromator\nd Spacing"
	pack $frame.d_spacing_label -side left

	frame $frame.sep3 -width 1 -borderwidth 1
	pack $frame.sep3 -side left -expand yes -fill y -padx 5

	label $frame.d_spacing_value -text "---" -fg blue
	pack $frame.d_spacing_value -side left

	frame $frame.sep4 -width 1 -borderwidth 1
	pack $frame.sep4 -side left -expand yes -fill y -padx 5

	label $frame.d_spacing_units -text "A"
	pack $frame.d_spacing_units -side left
}

############################################################################

proc create_slit_frame { frame } {

	set row 0

	label $frame.current_label -text Current -width 10
	label $frame.desired_label -text Desired

	grid $frame.current_label -row $row -column 1
	grid $frame.desired_label -row $row -column 3

	incr row

	label $frame.vert_label -text "Vertical"
	label $frame.vert_value -text "---" -fg blue
	label $frame.vert_units1 -text "um    "
	label $frame.vert_units2 -text "um"

	iwidgets::entryfield $frame.vert_desired -width 10 \
		-validate { iwidgets::Entryfield::real %P }

	grid $frame.vert_label   -row $row -column 0 -sticky w
	grid $frame.vert_value   -row $row -column 1 -sticky e
	grid $frame.vert_units1  -row $row -column 2 -sticky w
	grid $frame.vert_desired -row $row -column 3
	grid $frame.vert_units2  -row $row -column 4 -sticky w

	incr row

	label $frame.horiz_label -text "Horizontal"
	label $frame.horiz_value -text "---" -fg blue
	label $frame.horiz_units1 -text "um    "
	label $frame.horiz_units2 -text "um"

	iwidgets::entryfield $frame.horiz_desired -width 10 \
		-validate { iwidgets::Entryfield::real %P }

	grid $frame.horiz_label   -row $row -column 0 -sticky w
	grid $frame.horiz_value   -row $row -column 1 -sticky e
	grid $frame.horiz_units1  -row $row -column 2 -sticky w
	grid $frame.horiz_desired -row $row -column 3
	grid $frame.horiz_units2  -row $row -column 4 -sticky w
}

proc move_slits { frame } {

	global callback_records

	set horiz_requested [ $frame.horiz_desired get ]

	if { $horiz_requested != "" } {
		$callback_records(bh_size) move_absolute $horiz_requested
	}

	set vert_requested [ $frame.vert_desired get ]

	if { $vert_requested != "" } {
		$callback_records(bv_size) move_absolute $vert_requested
	}
}

