#
#  Copyright 1999 Illinois Institute of Technology
#
#  See the file "LICENSE" for information on usage and redistribution
#  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

proc create_mad_bottom_window { toplevel record_list } {

	create_mad_grid $toplevel $record_list

	frame $toplevel.sep2 -height 2 -borderwidth 1 -relief sunken
	pack $toplevel.sep2 -side top -expand yes -fill x -pady 5

	create_mad_buttonrow $toplevel $record_list
}

proc create_mad_grid { toplevel record_list } {

	set bottom $toplevel.bottom

	frame $bottom
	pack $bottom -side top

	set grid $bottom.grid

	frame $grid
	pack $grid -side top

	#
	# Please note in the following code that it is intentional
	# that the items in the "High" columns ( columns 8 and 9 ) are not
	# mapped onto the screen.  The value of $grid.num_wavelengths
	# is what determines whether or not the "High" row is displayed.
	#

	set entrywidth 10

	set row 0

	iwidgets::optionmenu $grid.num_wavelengths \
		-labelpos e -labeltext "Wavelengths:" \
		-command [ code mad_wavelength_select $grid ]

	$grid.num_wavelengths insert end Three Four
	$grid.num_wavelengths select Three

	label $grid.peak_label -text Peak
	label $grid.edge_label -text Edge
	label $grid.low_label  -text Low
	label $grid.high_label -text High

	grid $grid.num_wavelengths -row $row -column 1
	grid $grid.peak_label      -row $row -column 2
	grid $grid.edge_label      -row $row -column 4
	grid $grid.low_label       -row $row -column 6

	#-----

	incr row

	label $grid.wavelength_label      -text "Wavelength:"
	label $grid.peak_wavelength       -text "---" -fg blue
	label $grid.peak_wavelength_units -text A
	label $grid.edge_wavelength       -text "---" -fg blue
	label $grid.edge_wavelength_units -text A

	iwidgets::entryfield $grid.low_wavelength -width $entrywidth
	label $grid.low_wavelength_units  -text A

	$grid.low_wavelength configure -command \
		[ code chooch_display_far_params $toplevel low wavelength ]

	iwidgets::entryfield $grid.high_wavelength -width $entrywidth
	label $grid.high_wavelength_units -text A

	$grid.high_wavelength configure -command \
		[ code chooch_display_far_params $toplevel high wavelength ]

	grid $grid.wavelength_label       -row $row -column 1 -sticky e
	grid $grid.peak_wavelength        -row $row -column 2
	grid $grid.peak_wavelength_units  -row $row -column 3
	grid $grid.edge_wavelength        -row $row -column 4
	grid $grid.edge_wavelength_units  -row $row -column 5
	grid $grid.low_wavelength         -row $row -column 6
	grid $grid.low_wavelength_units   -row $row -column 7

	#-----

	incr row

	label $grid.energy_label      -text "Energy:"
	label $grid.peak_energy       -text "---" -fg blue
	label $grid.peak_energy_units -text eV
	label $grid.edge_energy       -text "---" -fg blue
	label $grid.edge_energy_units -text eV

	iwidgets::entryfield $grid.low_energy -width $entrywidth
	label $grid.low_energy_units  -text eV

	$grid.low_energy configure -command \
		[ code chooch_display_far_params $toplevel low energy ]

	iwidgets::entryfield $grid.high_energy -width $entrywidth
	label $grid.high_energy_units -text eV

	$grid.high_energy configure -command \
		[ code chooch_display_far_params $toplevel high energy ]

	grid $grid.energy_label       -row $row -column 1 -sticky e
	grid $grid.peak_energy        -row $row -column 2
	grid $grid.peak_energy_units  -row $row -column 3
	grid $grid.edge_energy        -row $row -column 4
	grid $grid.edge_energy_units  -row $row -column 5
	grid $grid.low_energy         -row $row -column 6
	grid $grid.low_energy_units   -row $row -column 7

	#-----

	incr row

	label $grid.fprime_label      -text "f prime:"
	label $grid.peak_fprime       -text "---" -fg blue
	label $grid.edge_fprime       -text "---" -fg blue
	label $grid.low_fprime        -text "---" -fg blue
	label $grid.high_fprime       -text "---" -fg blue

	grid $grid.fprime_label       -row $row -column 1 -sticky e
	grid $grid.peak_fprime        -row $row -column 2
	grid $grid.edge_fprime        -row $row -column 4
	grid $grid.low_fprime         -row $row -column 6

	#-----

	incr row

	label $grid.fdblprime_label      -text "f double prime:"
	label $grid.peak_fdblprime       -text "---" -fg blue
	label $grid.edge_fdblprime       -text "---" -fg blue
	label $grid.low_fdblprime        -text "---" -fg blue
	label $grid.high_fdblprime       -text "---" -fg blue

	grid $grid.fdblprime_label       -row $row -column 1 -sticky e
	grid $grid.peak_fdblprime        -row $row -column 2
	grid $grid.edge_fdblprime        -row $row -column 4
	grid $grid.low_fdblprime         -row $row -column 6

	#-----

	frame $bottom.button_sep -height 2 -borderwidth 1
	pack $bottom.button_sep -side top -expand yes -fill x -pady 1

	button $bottom.calc_fvalues \
		-text "Calculate\nf prime, f double prime" \
		-command [ code calc_fvalues $toplevel ]

	pack $bottom.calc_fvalues
}

proc mad_wavelength_select { grid } {

	set selection [ $grid.num_wavelengths get ]

	switch -exact -- $selection {
		Three { mad_wavelength_hide_high_column $grid }

		Four { mad_wavelength_show_high_column $grid }

		default { tk_dialog .unexpected_wavelength_selection_dialog \
			"Unexpected number of wavelengths" \
		"Should have 'Three' or 'Four' wavelengths, not '$selection'" \
			error 0 "OK" }
	}
}

proc mad_wavelength_hide_high_column { grid } {

	grid forget $grid.high_label
	grid forget $grid.high_wavelength
	grid forget $grid.high_wavelength_units
	grid forget $grid.high_energy
	grid forget $grid.high_energy_units
	grid forget $grid.high_fprime
	grid forget $grid.high_fdblprime

}

proc mad_wavelength_show_high_column { grid } {

	grid $grid.high_label            -row 0 -column 8
	grid $grid.high_wavelength       -row 1 -column 8
	grid $grid.high_wavelength_units -row 1 -column 9
	grid $grid.high_energy           -row 2 -column 8
	grid $grid.high_energy_units     -row 2 -column 9
	grid $grid.high_fprime           -row 3 -column 8
	grid $grid.high_fdblprime        -row 4 -column 8

}

proc create_mad_buttonrow { toplevel record_list } {

	set buttonrow $toplevel.buttonrow

	frame $buttonrow
	pack $buttonrow -side top

	button $buttonrow.print -text "Print\nWindow" \
				-command [ code print_window $toplevel ]

	pack $buttonrow.print -side left

	frame $buttonrow.sep1 -width 1 -borderwidth 1
	pack $buttonrow.sep1 -side left -expand yes -fill y -padx 20

	button $buttonrow.close -text Close -command [ code destroy $toplevel ]
	pack $buttonrow.close -side left
}

